\name{extract_plot}
\alias{extract_plot}
\title{Extraction of Biclusters and Plotting of the Results}
\description{

  \code{extract_plot}: \R implementation of  \code{extract_plot}.
}
\usage{

extract_plot(X,L,Z,thresZ=0.5,ti,thresL=NULL,Y=NULL,x11b=TRUE,norm=1,center=2)

}
\arguments{
  \item{X}{original data matrix.}
  \item{L}{loading, left matrix.}
  \item{Z}{factor, right matrix.}
  \item{thresZ}{threshold for sample belonging to bicluster (default 0.5).}
  \item{thresL}{threshold for loading belonging to bicluster (estimated
    if not given).}
  \item{ti}{plot title.}
  \item{Y}{noise free data matrix.}
  \item{x11b}{plot on screen.}
  \item{norm}{data normalization: default = 1 (yes), 0 (no).}
  \item{center}{data centering:  1 (mean), 2
    (median), > 2 (mode), 0 (no); default 2.}
}
\details{

  Essentially the model is the sum of
  outer products of vectors. The number of summands  \eqn{p}
  is the number of biclusters.


  \deqn{X \ = \ L \ Z \ + \ U}

  \deqn{X \ = \ \sum_{i=1}^{p} L_i \
                   \left(Z_i \right)^T \ + \ U}

   The hidden dimension \eqn{p} is used for kmeans clustering of
   \eqn{L_i} and \eqn{Z_i }.

   The \eqn{L_i } and \eqn{Z_i } are used to
   extract the bicluster \eqn{i}, where a threshold determines which
   observations and which samples belong the the bicluster.

   The method produces a couple of plots given below.

   \code{Plots:}
   \dQuote{Y}: noise free data (if available),
   \dQuote{X}: data,
   \dQuote{LZ}: reconstructed data,
   \dQuote{LZ-X}: error,
   \dQuote{abs(Z)}: absolute factors,
   \dQuote{abs(L)}: absolute loadings,
   \dQuote{abs(nL)}: absolute loadings normalized,
   \dQuote{abs(nZ)}: absolute factors normalized,
   \dQuote{nZ*pmZ}: factors sorted,
   \dQuote{pmL*nL}: loadings sorted,
   \dQuote{pmL*L*z*pmZ}: reconstructed matrix sorted,
   \dQuote{pmL*X*pmZ}: original matrix sorted.

   In above plots the matrix \eqn{L } and the matrix
   \eqn{Z } are sorted. For sorting first \code{kmeans} is
   on the \eqn{p} dimensional space is performed and then the vectors
   which belong to the same cluster are put together in the sorting.
   This sorting is made for visualization but in general
   it is not possible to
   visualize all biclusters as blocks if  they overlap.


   In \code{bic} the biclusters are extracted according to the
   largest absolute values of the component \eqn{i}, i.e.
   the largest values of \eqn{L_i } and the
   largest values of \eqn{Z_i }. The factors \eqn{ Z_i}
   are normalized to variance 1.

   The components of \code{bic} are
   \code{bin}, \code{bixv},
   \code{bixn}, \code{biypv}, \code{biypn}, \code{biynv},
   and \code{biynn}.
   \code{bin} gives the size of the bicluster: number observations,
   number positive samples, number negative samples.
   \code{bixv} gives the values of the observations that have absolute
   values above a threshold. They are sorted and
   \code{bixn} gives  their names (e.g. gene names).
   \code{biypv} gives the values of the samples that have
   values above a threshold. They are sorted and
   \code{biypn} gives  their names (e.g. sample names).
   \code{biynv} gives the values of the samples that have
   values below this threshold. They are sorted and
   \code{biynn} gives  their names (e.g. sample names).

   That means the samples are divided into two groups where one group
   shows large positive values and the other group has negative values
   with large absolute values. That means a observation pattern can be
   switched on or switched off relative to the average value.

   \code{numn} gives the indexes of \code{bic} with components:
   \code{numn1} = \code{bix} ,\code{numn2}  =  \code{biyp}, and
   \code{numn3} = \code{biyn}.

   The kmeans clusters are given by \code{biclust} with
   components \code{biclustx} (the clustered observations)
   and \code{biclusty} (the clustered samples).

  Implementation in \R.
}
\value{
  \item{bic}{extracted biclusters.}
  \item{numn}{indexes for the extracted biclusters.}
  \item{biclust}{clusters of kmeans clustering.}
  \item{pmZ}{permutation matrix of \eqn{z} from kmeans clustering.}
  \item{pmL}{permutation matrix of \eqn{\Lambda} from kmeans clustering.}
  \item{nL}{normalized loadings (left matrix).}
  \item{nZ}{normalized factors (right matrix).}
  \item{Xord}{sorted original matrix according to kmeans on \eqn{Z} and
  kmeans on \eqn{\Lambda}.}
    }
\seealso{
\code{\link{fabi}},
\code{\link{fabia}},
\code{\link{fabiap}},
\code{\link{fabias}},
\code{\link{fabiasp}},
\code{\link{mfsc}},
\code{\link{nmfdiv}},
\code{\link{nmfeu}},
\code{\link{nmfsc}},
\code{\link{nprojfunc}},
\code{\link{projfunc}},
\code{\link{estimateMode}},
\code{\link{make_fabi_data}},
\code{\link{make_fabi_data_blocks}},
\code{\link{make_fabi_data_pos}},
\code{\link{make_fabi_data_blocks_pos}},
\code{\link{extract_bic}},
\code{\link{myImagePlot}},
\code{\link{PlotBicluster}},
\code{\link{Breast_A}},
\code{\link{DLBCL_B}},
\code{\link{Multi_A}},
\code{\link{fabiaDemo}},
\code{\link{fabiaVersion}}
}
\author{Sepp Hochreiter}
\examples{

#---------------
# TEST
#---------------

dat <- make_fabi_data_blocks(n = 100,l= 50,p = 3,f1 = 5,f2 = 5,
  of1 = 5,of2 = 10,sd_noise = 3.0,sd_z_noise = 0.2,mean_z = 2.0,
  sd_z = 1.0,sd_l_noise = 0.2,mean_l = 3.0,sd_l = 1.0)

X <- dat[[1]]
Y <- dat[[2]]
X <- X- rowMeans(X)
XX <- (1/ncol(X))*tcrossprod(X)
dXX <- 1/sqrt(diag(XX)+0.001*as.vector(rep(1,nrow(X))))
X <- dXX*X



resEx <- fabia(X,20,0.3,1.0,1.0,3)

rEx <- extract_plot(X,resEx$L,resEx$Z,ti="FABIA",Y=Y,x11b=FALSE)

rEx$bic[1,]
rEx$bic[2,]
rEx$bic[3,]
rEx$biclust[1,]
rEx$biclust[2,]
rEx$biclust[3,]


\dontrun{
#---------------
# DEMO1
#---------------

dat <- make_fabi_data_blocks(n = 1000,l= 100,p = 10,f1 = 5,f2 = 5,
  of1 = 5,of2 = 10,sd_noise = 3.0,sd_z_noise = 0.2,mean_z = 2.0,
  sd_z = 1.0,sd_l_noise = 0.2,mean_l = 3.0,sd_l = 1.0)

X <- dat[[1]]
Y <- dat[[2]]


resToy <- fabia(X,200,0.4,1.0,1.0,13)

rToy <- extract_plot(X,resToy$L,resToy$Z,ti="FABIA",Y=Y)

#---------------
# DEMO2
#---------------

data(Breast_A)

X <- as.matrix(XBreast)

resBreast <- fabia(X,200,0.1,1.0,1.0,5)

rBreast <- extract_plot(X,resBreast$L,resBreast$Z,ti="FABIA Breast cancer(Veer)")

#sorting of predefined labels
CBreast%*%rBreast$pmZ
}
}
\keyword{hplot}
\concept{biclustering}
\concept{sparse coding}
\concept{sparse matrix factorization}
\concept{non-negative matrix factorization}
